#include<stdlib.h>
#include<conio.h>
#include<math.h>
#include<stdio.h>
#include<stdlib.h>

/* ****************************************************************
	 Brute force program to evaluate a convergent integral
     		            with infinity limits.
	 After getting familiar with functioning of the program, you should
	 fix the number of steps and put the procedures in a larger program.
	 Have fun !
	 -------------------- D. Ciurchea 2001 ---------------
	 Algorithm:
	 - Machine significance gives the value of the function
		 for which furher summation is worthless.
	 - The abscissa for this cut off value is found by some
		 algorithm of finding the root of the nonlinear equation:

			 f(x)-CutOffValue=0

 	   Here, a derivative free version of Newton's method is used.
	 - Use of Simpson's rule to perform integration. Error evaluation
	   is made by doubling the integration step.
	 - If the integration error is not satisfactory, the integration
	   step is increased until the mismatch disappears.
	 **************************************************************** */


double f(double x, double CutOffValue) /* The function to analyze */
{                              /*Normally, CutOffValue should be 0*/
double ff,pi=4*atan(1.0);
	/*
	 *	Here is the Gauss function, for which the integral
	 *	from 0 to +infinity should yield 0.5
	 */
ff = exp (-x * x / 2 )/ sqrt ( 2 * pi ) - CutOffValue ;
return ff;
}

double MAX(double x,double y)/* Maximum of two real numbers */
{
	if (x>y) return x;
            else return y;
}

double precision()/* Find machine precision (significance)*/
{
double reps=1.0 ;
	 while(1.0 + reps != 1.0 ) reps /= 2 ;
	 reps *= 2 ;
return reps;
}

	/* Derivative free version of Newton's method to find *\
	\* the root of a nonlinear equation                   */
double Newton
		(double x,  // The variable 
		 double reps, // Machine precision                  
		 double CutOffValue)
{
	double xnm1,xn=x,eps=sqrt(reps),ff,pas,der;
	while ( xnm1 != xn )
	{
		xnm1 = xn;
		pas = MAX(xnm1,0.1)*10*eps;
		ff = f(xnm1,CutOffValue);
		der = (f(xnm1+pas,CutOffValue)- ff)/pas;
		xn = xnm1 - ff/der;
	}
	return xn;
}

	/* Simnpson's rule for calculating a *\
	\*        definite integral          */

double Simpson(double x_min,double x_max,int N_steps)
{
	int i;
	double s1,s2,step,hh, FinalValue;
	step=(x_max-x_min)/2/N_steps;
	for (i=1,s1=0,s2=0;i<=N_steps;i++)
	{
		hh=2*i*step;
		s1+=f(x_min+hh-step,0);
		s2+=f(x_min+hh,0);
	}
	FinalValue = step / 3 * (f(x_min,0)-f(x_max,0)+4 * s1 + 2 * s2 );
	return ( FinalValue );
}

main ()
{
	/* ******  Declarations of variables ***** */
	double pi=4*atan(1.0),x=1,reps,CutOffValue;
	double DesiredPrecision=1e-9; /*desired precision
                                in calculation of the integral*/
	double x_min=0;  /* Inferior limit of the integral*/
	double t,tt;     /*Values of the integral for a given step 
                   and its double respectively*/
	int n=2;         /*initial guess for the number of steps
                   for the evaluation of the integral*/
	double error;    /*Error in evaluation of the integral*/
	int counter=1;   /* counter for theincrease in the nr.
                    of integration steps*/

			/***********    Main program   **********/
	//clrscr();
	reps = precision();
	DesiredPrecision = 1e-10 ;/*Put your claims here*/
	CutOffValue = reps;
	/*
	  Normally, the value of the machine significance should be
	  equivalent to 0 in a summation, but should supply an ultimate 
	  CUT OFF for convergent integrals. The value is obviously 
	  exagerately small. Normally, 10*reps, 1e-7 or 1e-5 as 
	  absolute values should suffice. 
	*/
	/* X value for the cut off of the integral */
	x = Newton (x,reps,CutOffValue);
	printf ("\nThe Cut_Off_Value is x= %lf ",x);
one:
	t=Simpson(x_min,x,n);
	tt=Simpson(x_min,x,n+n);
	error=fabs(t-tt)/15 ;
	if ( error>DesiredPrecision )
	{
		counter++;
		n*=2;/*or other efficient increse in the number of steps*/
		printf ("\t\a\nNumber of steps=%d",n);
		printf ("\n Evaluation %d: error=%lf old= %lf  new=%lf",
							counter, error, t,tt);
		goto one;
	}
	printf ("\nAttempt %d: Evaluated error=%e old= %10.8lf  new=%10.8lf",
																					counter, error, t,tt);
	getch ();
	return 0 ;
}